@@ -237,7 +237,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 {
 	EBookBackendKolab *self = NULL;
 	EBookBackendKolabPrivate *priv = NULL;
-	ESourceRegistry *registry = NULL;
 	ESource *esource = NULL;
 	ESourceKolabFolder *extension = NULL;
 	CamelKolabIMAPXSettings *kolab_settings = NULL;
@@ -252,7 +251,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	gchar *user_at_server = NULL;
 	gchar *tmp_key = NULL;
 	gboolean online = FALSE;
-	gboolean auth_required = FALSE;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
 
@@ -267,8 +265,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 
 	g_mutex_lock (&koma_objects_lock);
 
-	registry = e_book_backend_get_registry (E_BOOK_BACKEND (backend));
-
 	esource = e_backend_get_source (E_BACKEND (backend));
 	if (! E_IS_SOURCE (esource)) {
 		tmp_err = e_data_book_create_error (E_DATA_BOOK_STATUS_OTHER_ERROR,
@@ -300,8 +296,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	                                   (gpointer *) &tmp_koma);
 
 	if (ok) {
-		gchar *passwd = NULL;
-
 		/* There already is a KoMA instance for user_at_server.
 		 * Use it and return
 		 */
@@ -313,18 +307,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 		                                     NULL,
 		                                     foldername,
 		                                     &sync_value);
-		/* We may find that we do not yet have a password set.
-		 * In case we're requested to go online, we will need
-		 * one, so we need to get authentication data first.
-		 */
-		passwd = g_strdup (kolab_settings_handler_get_char_field (ksettings,
-		                                                          KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD,
-		                                                          NULL));
-		if (passwd == NULL) {
-			auth_required = TRUE;
-			goto exit;
-		}
-		g_free (passwd);
 
 		(void) kolab_util_backend_deploy_mode_by_backend (priv->book_koma,
 		                                                  online,
@@ -376,6 +358,7 @@ book_backend_kolab_open (EBookBackendSync *backend,
 
 	/* configure and bring up KolabMailAccess instance */
 	ok = kolab_mail_access_configure (priv->book_koma,
+	                                  E_BACKEND (backend),
 	                                  ksettings,
 	                                  &tmp_err);
 	if (! ok)
@@ -387,15 +370,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 	if (! ok)
 		goto exit;
 
-	/* If we are here (just created a fresh KolabMailAccess),
-	 * we did not yet receive authentication data. Though we
-	 * do not need it if we want to stay in offline mode, once
-	 * we're asked to go online, we'll need the credentials
-	 * (password in our case) anyway, so we can just request
-	 * it once the book gets opened.
-	 */
-	auth_required = TRUE;
-
  exit:
 
 	if (ksettings != NULL)
@@ -417,18 +391,6 @@ book_backend_kolab_open (EBookBackendSync *backend,
 		return;
 	}
 
-	/* authenticate now if required */
-	if (auth_required) {
-		ESourceAuthenticator *authenticator;
-
-		authenticator = E_SOURCE_AUTHENTICATOR (priv->book_koma);
-
-		ok = e_source_registry_authenticate_sync (
-			registry, esource, authenticator, cancellable, error);
-		if (!ok)
-			return;
-	}
-
 	/* all good, notify that we're open for business */
 	ok = book_backend_kolab_notify_opened (self, &tmp_err);
 	if (! ok) {
