@@ -261,7 +261,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 {
 	ECalBackendKolab *self = NULL;
 	ECalBackendKolabPrivate *priv = NULL;
-	ESourceRegistry *registry = NULL;
 	ESource *esource = NULL;
 	ESourceKolabFolder *extension = NULL;
 	CamelKolabIMAPXSettings *kolab_settings = NULL;
@@ -279,7 +278,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	gchar *user_at_server = NULL;
 	gchar *tmp_key = NULL;
 	gboolean online = FALSE;
-	gboolean auth_required = FALSE;
 	gboolean ok = FALSE;
 	GError *tmp_err = NULL;
 
@@ -294,8 +292,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 
 	g_mutex_lock (&koma_objects_lock);
 
-	registry = e_cal_backend_get_registry (E_CAL_BACKEND (backend));
-
 	icalkind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
 	/* TODO: this has to be set according to the get_kind() method */
 	switch (icalkind) {
@@ -360,8 +356,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	                                   (gpointer *) &tmp_koma);
 
 	if (ok) {
-		gchar *passwd = NULL;
-
 		/* There already is a KoMA instance for user_at_server.
 		 * Use it and return
 		 */
@@ -373,18 +367,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 		                                     NULL,
 		                                     foldername,
 		                                     &sync_value);
-		/* We may find that we do not yet have a password set.
-		 * In case we're requested to go online, we will need
-		 * one, so we need to get authentication data first.
-		 */
-		passwd = g_strdup (kolab_settings_handler_get_char_field (ksettings,
-		                                                          KOLAB_SETTINGS_HANDLER_CHAR_FIELD_KOLAB_USER_PASSWORD,
-		                                                          NULL));
-		if (passwd == NULL) {
-			auth_required = TRUE;
-			goto exit;
-		}
-		g_free (passwd);
 
 		(void) kolab_util_backend_deploy_mode_by_backend (priv->cal_koma,
 		                                                  online,
@@ -436,6 +418,7 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 
 	/* configure and bring up KolabMailAccess instance */
 	ok = kolab_mail_access_configure (priv->cal_koma,
+	                                  E_BACKEND (backend),
 	                                  ksettings,
 	                                  &tmp_err);
 	if (! ok)
@@ -453,15 +436,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 	 */
 	priv->user_email = g_strdup (user_at_server);
 
-	/* If we are here (just created a fresh KolabMailAccess),
-	 * we did not yet receive authentication data. Though we
-	 * do not need it if we want to stay in offline mode, once
-	 * we're asked to go online, we'll need the credentials
-	 * (password in our case) anyway, so we can just request
-	 * it once the book gets opened.
-	 */
-	auth_required = TRUE;
-
  exit:
 
 	if (ksettings != NULL)
@@ -483,18 +457,6 @@ cal_backend_kolab_open (ECalBackendSync *backend,
 		return;
 	}
 
-	/* authenticate now if required */
-	if (auth_required) {
-		ESourceAuthenticator *authenticator;
-
-		authenticator = E_SOURCE_AUTHENTICATOR (priv->cal_koma);
-
-		ok = e_source_registry_authenticate_sync (
-			registry, esource, authenticator, cancellable, error);
-		if (!ok)
-			return;
-	}
-
 	/* all good, notify that we're open for business */
 	ok = cal_backend_kolab_notify_opened (self, &tmp_err);
 	if (! ok) {
@@ -1228,6 +1190,8 @@ cal_backend_kolab_remove_objects (ECalBackendSync *backend,
 	ECalComponent *oldcomp = NULL;
 	ECalComponent *newcomp = NULL;
 	ECalComponent *oldtz = NULL;
+	ECalComponent *oc_data = NULL;
+	ECalComponent *nc_data = NULL;
 	ECalComponentId *id = NULL;
 	const gchar *uid, *rid;
 	KolabSettingsHandler *ksettings = NULL;
@@ -1381,10 +1345,15 @@ cal_backend_kolab_remove_objects (ECalBackendSync *backend,
 		}
 	}
 
+	if (*old_components != NULL)
+		oc_data = (*old_components)->data;
+	if (*new_components = NULL)
+		nc_data = (*new_components)->data;
+	
 	e_cal_backend_notify_component_removed (E_CAL_BACKEND (backend),
 	                                        id,
-	                                        (*old_components)->data,
-	                                        (*new_components)->data);
+	                                        oc_data,
+	                                        nc_data);
 	e_cal_component_free_id (id);
 }
 
