@@ -82,10 +82,6 @@ kolab_util_calendar_extract (icalcomponent *icalcomp,
                              ECalComponent **tzcomp,
                              GError **error);
 
-#if 0
-ECalBackendSyncStatus kolab_util_calendar_map_error (GError *error);
-#endif
-
 void
 kolab_util_calendar_err_to_edb_err (GError **e_err,
                                     const GError *k_err,
