@@ -421,4 +421,25 @@ e_kolab_folder_permissions_ui_free (KolabFolderPermUIData *uidata)
 	g_free (uidata);
 }
 
+void
+e_kolab_folder_permissions_ui_update_from_uidata (KolabFolderPermUIData *uidata)
+{
+	g_return_if_fail (uidata != NULL);
+
+	g_warning ("%s: FIXME implement me", __func__);
+}
+
+gboolean
+e_kolab_folder_permissions_ui_query_acl (KolabFolderPermUIData *uidata,
+                                         GCancellable *cancellable,
+                                         GError **err)
+{
+	g_return_val_if_fail (uidata != NULL, FALSE);
+	/* cancellable may be NULL */
+	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+
+	g_warning ("%s: FIXME implement me", __func__);
+	return TRUE;
+}
+
 /*----------------------------------------------------------------------------*/
