@@ -71,7 +71,7 @@ camel_imapx_metadata_attrib_free (CamelImapxMetadataAttrib *ma)
 }
 
 static void
-camel_imapx_metadata_attrib_gdestroy (gpointer data)
+imapx_metadata_attrib_gdestroy (gpointer data)
 {
 	CamelImapxMetadataAttrib *ma = (CamelImapxMetadataAttrib *)data;
 	camel_imapx_metadata_attrib_free (ma);
@@ -86,7 +86,7 @@ camel_imapx_metadata_entry_new (void)
 	me->attributes = g_hash_table_new_full (g_str_hash,
 	                                        g_str_equal,
 	                                        g_free,
-	                                        camel_imapx_metadata_attrib_gdestroy);
+	                                        imapx_metadata_attrib_gdestroy);
 	return me;
 }
 
@@ -103,7 +103,7 @@ camel_imapx_metadata_entry_free (CamelImapxMetadataEntry *me)
 }
 
 static void
-camel_imapx_metadata_entry_gdestroy (gpointer data)
+imapx_metadata_entry_gdestroy (gpointer data)
 {
 	CamelImapxMetadataEntry *me = (CamelImapxMetadataEntry *)data;
 	camel_imapx_metadata_entry_free (me);
@@ -118,7 +118,7 @@ camel_imapx_metadata_annotation_new (void)
 	man->entries = g_hash_table_new_full (g_str_hash,
 	                                      g_str_equal,
 	                                      g_free,
-	                                      camel_imapx_metadata_entry_gdestroy);
+	                                      imapx_metadata_entry_gdestroy);
 	return man;
 }
 
@@ -135,7 +135,7 @@ camel_imapx_metadata_annotation_free (CamelImapxMetadataAnnotation *man)
 }
 
 static void
-camel_imapx_metadata_annotation_gdestroy (gpointer data)
+imapx_metadata_annotation_gdestroy (gpointer data)
 {
 	CamelImapxMetadataAnnotation *man = (CamelImapxMetadataAnnotation *)data;
 	camel_imapx_metadata_annotation_free (man);
@@ -160,7 +160,7 @@ camel_imapx_metadata_new (camel_imapx_metadata_proto_t proto,
 	md->mboxes = g_hash_table_new_full (g_str_hash,
 	                                    g_str_equal,
 	                                    g_free,
-	                                    camel_imapx_metadata_annotation_gdestroy);
+	                                    imapx_metadata_annotation_gdestroy);
 	return md;
 }
 
@@ -181,27 +181,24 @@ camel_imapx_metadata_free (CamelImapxMetadata *md)
 }
 
 CamelImapxMetadata*
-camel_imapx_metadata_resect (CamelImapxMetadata **md)
+camel_imapx_metadata_resect (CamelImapxMetadata *md)
 {
 	CamelImapxMetadata *tmp_md = NULL;
-	GMutex tmp_lock;
+	GHashTable *mboxes = NULL;
 
 	if (md == NULL)
 		return NULL;
 
 	/* (acquire md lock) */
-	g_mutex_lock (&((*md)->md_lock));
+	g_mutex_lock (&(md->md_lock));
 
-	tmp_md = *md;
-	*md = camel_imapx_metadata_new (tmp_md->proto, FALSE);
-
-	/* (swap lock pointers of md and tmp_md) */
-	tmp_lock = tmp_md->md_lock;		/* locked   */
-	tmp_md->md_lock = (*md)->md_lock;	/* unlocked */
-	(*md)->md_lock = tmp_lock;		/* locked   */
+	tmp_md = camel_imapx_metadata_new (md->proto, FALSE);
+	mboxes = md->mboxes;
+	md->mboxes = tmp_md->mboxes;
+	tmp_md->mboxes = mboxes;
 
 	/* (release md lock) */
-	g_mutex_unlock (&((*md)->md_lock));
+	g_mutex_unlock (&(md->md_lock));
 
 	return tmp_md;
 }
@@ -209,20 +206,48 @@ camel_imapx_metadata_resect (CamelImapxMetadata **md)
 camel_imapx_metadata_proto_t
 camel_imapx_metadata_get_proto (CamelImapxMetadata *md)
 {
-	camel_imapx_metadata_proto_t mp = CAMEL_IMAPX_METADATA_PROTO_INVAL;
+	camel_imapx_metadata_proto_t proto = CAMEL_IMAPX_METADATA_PROTO_INVAL;
 
-	if (md == NULL)
-		return CAMEL_IMAPX_METADATA_PROTO_INVAL;
+	g_return_val_if_fail (md != NULL, CAMEL_IMAPX_METADATA_PROTO_INVAL);
 
 	/* (acquire md lock) */
 	g_mutex_lock (&(md->md_lock));
 
-	mp = md->proto;
+	proto = md->proto;
 
-	/* (acquire md lock) */
+	/* (release md lock) */
 	g_mutex_unlock (&(md->md_lock));
 
-	return mp;
+	return proto;
+}
+
+gboolean
+camel_imapx_metadata_set_proto (CamelImapxMetadata *md,
+                                camel_imapx_metadata_proto_t proto)
+{
+	gboolean ok = FALSE;
+
+	g_return_val_if_fail (md != NULL, FALSE);
+	g_return_val_if_fail ((proto > CAMEL_IMAPX_METADATA_PROTO_INVAL) &&
+	                      (proto < CAMEL_IMAPX_METADATA_LAST_PROTO), FALSE);
+
+	/* (acquire md lock) */
+	g_mutex_lock (&(md->md_lock));
+
+	if (md->proto != CAMEL_IMAPX_METADATA_PROTO_INVAL) {
+		g_warning ("%s: Cannot reconfigure metadata protocol type",
+		           __func__);
+		goto exit;
+	}
+
+	md->proto = proto;
+	ok = TRUE;
+
+	/* (release md lock) */
+	g_mutex_unlock (&(md->md_lock));
+
+ exit:
+	return ok;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -265,10 +290,10 @@ camel_imapx_metadata_get_proto (CamelImapxMetadata *md)
  * nstring = string / "NIL"
  */
 static gboolean
-camel_imapx_metadata_parse_annotation_response (CamelImapxMetadata *md,
-                                                CamelIMAPXStream *is,
-                                                GCancellable *cancellable,
-                                                GError **err)
+imapx_metadata_parse_annotation_response (CamelImapxMetadata *md,
+                                          CamelIMAPXStream *is,
+                                          GCancellable *cancellable,
+                                          GError **err)
 {
 	gint tok = 0;
 	guint len = 0;
@@ -336,8 +361,7 @@ camel_imapx_metadata_parse_annotation_response (CamelImapxMetadata *md,
 	if (tok != '(') {
 		tmp_err = g_error_new (KOLAB_CAMEL_ERROR,
 		                       KOLAB_CAMEL_ERROR_GENERIC,
-		                       "%s: malformed annotation response (expected '(' after annotation name)",
-		                       __func__);
+		                       _("Malformed IMAP annotation response, expected '(' after annotation name"));
 		g_propagate_error (err, tmp_err);
 		return FALSE;
 	}
@@ -452,10 +476,10 @@ camel_imapx_metadata_parse_annotation_response (CamelImapxMetadata *md,
 }
 
 static gboolean
-camel_imapx_metadata_parse_metadata_response (CamelImapxMetadata *md,
-                                              CamelIMAPXStream *is,
-                                              GCancellable *cancellable,
-                                              GError **err)
+imapx_metadata_parse_metadata_response (CamelImapxMetadata *md,
+                                        CamelIMAPXStream *is,
+                                        GCancellable *cancellable,
+                                        GError **err)
 {
 	/* TODO implement me*/
 
@@ -488,16 +512,16 @@ camel_imapx_metadata_add_from_server_response (CamelImapxMetadata *md,
 
 	switch (md->proto) {
 	case CAMEL_IMAPX_METADATA_PROTO_ANNOTATEMORE:
-		parse_ok = camel_imapx_metadata_parse_annotation_response (md,
-		                                                           is,
-		                                                           cancellable,
-		                                                           &tmp_err);
+		parse_ok = imapx_metadata_parse_annotation_response (md,
+		                                                     is,
+		                                                     cancellable,
+		                                                     &tmp_err);
 		break;
 	case CAMEL_IMAPX_METADATA_PROTO_METADATA:
-		parse_ok = camel_imapx_metadata_parse_metadata_response (md,
-		                                                         is,
-		                                                         cancellable,
-		                                                         &tmp_err);
+		parse_ok = imapx_metadata_parse_metadata_response (md,
+		                                                   is,
+		                                                   cancellable,
+		                                                   &tmp_err);
 		break;
 	case CAMEL_IMAPX_METADATA_PROTO_INVAL:
 		/* (release md lock) */
