@@ -75,7 +75,10 @@ struct _CamelKolabIMAPXStoreClass {
 	CamelIMAPXExtdStoreClass parent_class;
 
 	gboolean (*set_folder_creation_type) (CamelKolabIMAPXStore *self,
-	                                      KolabFolderTypeID type_id);
+	                                      KolabFolderTypeID type_id,
+	                                      gboolean check_context);
+
+	KolabFolderTypeID (*get_folder_creation_type) (CamelKolabIMAPXStore *self);
 
 	gboolean (*set_folder_context) (CamelKolabIMAPXStore *self,
 	                                KolabFolderContextID context);
@@ -97,10 +100,14 @@ struct _CamelKolabIMAPXStoreClass {
 
 GType camel_kolab_imapx_store_get_type (void);
 
-/* Kolab extension: set type for newly created folders */
+/* Kolab extension: set/get type for newly created folders */
 gboolean
 camel_kolab_imapx_store_set_folder_creation_type (CamelKolabIMAPXStore *self,
-                                                  KolabFolderTypeID type_id);
+                                                  KolabFolderTypeID type_id,
+                                                  gboolean check_context);
+
+KolabFolderTypeID
+camel_kolab_imapx_store_get_folder_creation_type (CamelKolabIMAPXStore *self);
 
 /* Kolab extension: set the folder context (email, calendar, contacts (defaults to email)) */
 gboolean
