@@ -44,6 +44,7 @@
 #include <prtypes.h>
 
 #include <libedataserverui/libedataserverui.h>
+#include <libemail-engine/e-mail-authenticator.h>
 
 /* Kolab error reporting */
 #include <libekolabutil/kolab-util-error.h>
@@ -57,17 +58,18 @@ static gchar *nss_tok_pin = NULL; /* FIXME better solution for this */
 
 /*----------------------------------------------------------------------------*/
 
-typedef struct _CamelKolabSessionPrivate CamelKolabSessionPrivate;
 struct _CamelKolabSessionPrivate {
-	/* TODO these should be retrieved
-	 * from CamelKolabSettings instead
-	 */
-	gchar *passwd;
-
 	/* TODO get rid of this workaround */
 	gchar *nss_tok_pwd;
 
 	gboolean is_initialized;
+
+	GWeakRef backend;
+};
+
+enum {
+	PROP_0,
+	PROP_BACKEND
 };
 
 #define CAMEL_KOLAB_SESSION_PRIVATE(obj)  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CAMEL_TYPE_KOLAB_SESSION, CamelKolabSessionPrivate))
@@ -79,14 +81,57 @@ G_DEFINE_TYPE (CamelKolabSession, camel_kolab_session, CAMEL_TYPE_SESSION)
 static void
 camel_kolab_session_init (CamelKolabSession *self)
 {
-	CamelKolabSessionPrivate *priv = NULL;
+	self->priv = CAMEL_KOLAB_SESSION_PRIVATE (self);
+}
 
-	g_assert (CAMEL_IS_KOLAB_SESSION (self));
-	priv = CAMEL_KOLAB_SESSION_PRIVATE (self);
+static void
+camel_kolab_session_set_backend (CamelKolabSession *session,
+                                 EBackend *backend)
+{
+	g_return_if_fail (E_IS_BACKEND (backend));
 
-	priv->nss_tok_pwd = NULL;
-	priv->passwd = NULL;
-	priv->is_initialized = FALSE;
+	g_weak_ref_set (&session->priv->backend, backend);
+}
+
+static void
+camel_kolab_session_set_property (GObject *object,
+                                  guint property_id,
+                                  const GValue *value,
+                                  GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_BACKEND:
+			camel_kolab_session_set_backend (
+				CAMEL_KOLAB_SESSION (object),
+				g_value_get_object (value));
+			return;
+
+		default:
+			break;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+camel_kolab_session_get_property (GObject *object,
+                                  guint property_id,
+                                  GValue *value,
+                                  GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_BACKEND:
+			g_value_take_object (
+				value,
+				camel_kolab_session_ref_backend (
+				CAMEL_KOLAB_SESSION (object)));
+			return;
+
+		default:
+			break;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 }
 
 static void
@@ -109,8 +154,8 @@ camel_kolab_session_finalize (GObject *object)
 
 	if (priv->nss_tok_pwd != NULL)
 		g_free (priv->nss_tok_pwd);
-	if (priv->passwd)
-		g_free (priv->passwd);
+
+	g_weak_ref_set (&priv->backend, NULL);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (camel_kolab_session_parent_class)->finalize (object);
@@ -138,61 +183,6 @@ pk11_password (PK11SlotInfo *slot,
 /*----------------------------------------------------------------------------*/
 /* class functions */
 
-static gchar*
-kolab_session_get_password (CamelSession *self,
-                            CamelService *service,
-                            const gchar *prompt,
-                            const gchar *item,
-                            guint32 flags,
-                            GError **err)
-{
-	/* TODO rework to CamelSettings */
-
-	CamelKolabSession *myself = NULL;
-	CamelKolabSessionPrivate *priv = NULL;
-
-	g_assert (CAMEL_IS_KOLAB_SESSION (self));
-	g_assert (CAMEL_IS_KOLAB_IMAPX_STORE (service));
-	g_return_val_if_fail (err == NULL || *err == NULL, NULL);
-
-	(void)prompt;
-	(void)item;
-	(void)flags;
-
-	myself = CAMEL_KOLAB_SESSION (self);
-	priv = CAMEL_KOLAB_SESSION_PRIVATE (myself);
-
-	return g_strdup (priv->passwd);
-}
-
-static gboolean
-kolab_session_forget_password (CamelSession *self,
-                               CamelService *service,
-                               const gchar *item,
-                               GError **err)
-{
-	/* TODO rework to CamelSettings */
-
-	CamelKolabSession *myself = NULL;
-	CamelKolabSessionPrivate *priv = NULL;
-
-	g_assert (CAMEL_IS_KOLAB_SESSION (self));
-	g_assert (CAMEL_IS_KOLAB_IMAPX_STORE (service));
-	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
-
-	(void)item;
-
-	myself = CAMEL_KOLAB_SESSION (self);
-	priv = CAMEL_KOLAB_SESSION_PRIVATE (myself);
-
-	if (priv->passwd != NULL) {
-		g_free (priv->passwd);
-		priv->passwd = NULL;
-	}
-
-	return TRUE;
-}
-
 static gboolean
 kolab_session_authenticate_sync (CamelSession *session,
                                  CamelService *service,
@@ -200,11 +190,12 @@ kolab_session_authenticate_sync (CamelSession *session,
                                  GCancellable *cancellable,
                                  GError **error)
 {
+	CamelKolabSession *kolab_session;
+	ESourceAuthenticator *auth;
 	CamelServiceAuthType *authtype = NULL;
 	CamelAuthenticationResult result;
-	CamelProvider *provider;
-	const gchar *password;
-	guint32 password_flags;
+	EBackend *backend;
+	gboolean authenticated;
 	GError *local_error = NULL;
 
 	/* this is a dupe of mail_session_authenticate_sync()
@@ -218,7 +209,9 @@ kolab_session_authenticate_sync (CamelSession *session,
 	/* Do not chain up.  Camel's default method is only an example for
 	 * subclasses to follow.  Instead we mimic most of its logic here. */
 
-	provider = camel_service_get_provider (service);
+	/* Treat a mechanism name of "none" as NULL. */
+	if (g_strcmp0 (mechanism, "none") == 0)
+		mechanism = NULL;
 
 	/* APOP is one case where a non-SASL mechanism name is passed, so
 	 * don't bail if the CamelServiceAuthType struct comes back NULL. */
@@ -228,11 +221,13 @@ kolab_session_authenticate_sync (CamelSession *session,
 	/* If the SASL mechanism does not involve a user
 	 * password, then it gets one shot to authenticate. */
 	if (authtype != NULL && !authtype->need_password) {
-		result = camel_service_authenticate_sync (service, mechanism, cancellable, error);
+		result = camel_service_authenticate_sync (
+			service, mechanism, cancellable, error);
 		if (result == CAMEL_AUTHENTICATION_REJECTED)
-			g_set_error (error, CAMEL_SERVICE_ERROR,
-			             CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
-			             _("%s authentication failed"), mechanism);
+			g_set_error (
+				error, CAMEL_SERVICE_ERROR,
+				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+				_("%s authentication failed"), mechanism);
 		return (result == CAMEL_AUTHENTICATION_ACCEPTED);
 	}
 
@@ -253,9 +248,8 @@ kolab_session_authenticate_sync (CamelSession *session,
 		 *     detect errors. */
 		sasl = camel_sasl_new (service_name, mechanism, service);
 		if (sasl != NULL) {
-			success = camel_sasl_try_empty_password_sync (sasl,
-			                                              cancellable,
-			                                              &local_error);
+			success = camel_sasl_try_empty_password_sync (
+				sasl, cancellable, &local_error);
 			g_object_unref (sasl);
 		}
 
@@ -270,61 +264,19 @@ kolab_session_authenticate_sync (CamelSession *session,
 
 	g_clear_error (&local_error);
 
-	password_flags = CAMEL_SESSION_PASSWORD_SECRET;
+	kolab_session = CAMEL_KOLAB_SESSION (session);
+	backend = camel_kolab_session_ref_backend (kolab_session);
 
- retry:
-	password = camel_service_get_password (service);
+	auth = e_mail_authenticator_new (service, mechanism);
 
-	if (password == NULL) {
-		CamelSettings *settings;
-		CamelNetworkSettings *network_settings;
-		const gchar *host;
-		const gchar *user;
-		gchar *prompt;
-		gchar *new_passwd;
+	authenticated = e_backend_authenticate_sync (
+		backend, auth, cancellable, error);
 
-		settings = camel_service_ref_settings (service);
+	g_object_unref (auth);
 
-		network_settings = CAMEL_NETWORK_SETTINGS (settings);
-		host = camel_network_settings_get_host (network_settings);
-		user = camel_network_settings_get_user (network_settings);
+	g_object_unref (backend);
 
-		prompt = camel_session_build_password_prompt (provider->name,
-		                                              user,
-		                                              host);
-
-		g_object_unref (settings);
-
-		new_passwd = camel_session_get_password (session, service, prompt, "password",
-		                                         password_flags, &local_error);
-		camel_service_set_password (service, new_passwd);
-		password = camel_service_get_password (service);
-		g_free (new_passwd);
-
-		g_free (prompt);
-
-		if (local_error != NULL) {
-			g_propagate_error (error, local_error);
-			return FALSE;
-		}
-
-		if (password == NULL) {
-			g_set_error (error, CAMEL_SERVICE_ERROR,
-			             CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
-			             _("No password was provided"));
-			return FALSE;
-		}
-	}
-
-	result = camel_service_authenticate_sync (service, mechanism, cancellable, error);
-
-	if (result == CAMEL_AUTHENTICATION_REJECTED) {
-		password_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
-		camel_service_set_password (service, NULL);
-		goto retry;
-	}
-
-	return (result == CAMEL_AUTHENTICATION_ACCEPTED);
+	return authenticated;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -339,20 +291,55 @@ camel_kolab_session_class_init (CamelKolabSessionClass *klass)
 
 	g_type_class_add_private (klass, sizeof (CamelKolabSessionPrivate));
 
-	object_class->set_property = G_OBJECT_CLASS (parent_class)->set_property;
-	object_class->get_property = G_OBJECT_CLASS (parent_class)->get_property;
+	object_class->set_property = camel_kolab_session_set_property;
+	object_class->get_property = camel_kolab_session_get_property;
 	object_class->dispose = camel_kolab_session_dispose;
 	object_class->finalize = camel_kolab_session_finalize;
 
 	session_class->add_service = parent_class->add_service;
-	session_class->get_password = kolab_session_get_password;
-	session_class->forget_password = kolab_session_forget_password;
 	session_class->authenticate_sync = kolab_session_authenticate_sync;
+
+	g_object_class_install_property (
+		object_class,
+		PROP_BACKEND,
+		g_param_spec_object (
+			"backend",
+			"Backend",
+			"Backend used for authentication",
+			E_TYPE_BACKEND,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
 }
 
 /*----------------------------------------------------------------------------*/
 /* public API */
 
+CamelKolabSession *
+camel_kolab_session_new (EBackend *backend,
+                         const gchar *user_data_dir,
+                         const gchar *user_cache_dir)
+{
+	g_return_val_if_fail (E_IS_BACKEND (backend), NULL);
+	g_return_val_if_fail (user_data_dir != NULL, NULL);
+	g_return_val_if_fail (user_cache_dir != NULL, NULL);
+
+	return g_object_new (
+		CAMEL_TYPE_KOLAB_SESSION,
+		"backend", backend,
+		"user-data-dir", user_data_dir,
+		"user-cache-dir", user_cache_dir,
+		NULL);
+}
+
+EBackend *
+camel_kolab_session_ref_backend (CamelKolabSession *self)
+{
+	g_return_val_if_fail (CAMEL_IS_KOLAB_SESSION (self), NULL);
+
+	return g_weak_ref_get (&self->priv->backend);
+}
+
 gboolean
 camel_kolab_session_bringup (CamelKolabSession *self,
                              GCancellable *cancellable,
@@ -431,21 +418,4 @@ camel_kolab_session_set_token_pin (CamelKolabSession *self,
 	nss_tok_pin = g_strdup (pin);
 }
 
-void
-camel_kolab_session_set_password (CamelKolabSession *self,
-                                  const gchar *passwd)
-{
-	/* TODO rework to CamelSettings */
-
-	CamelKolabSessionPrivate *priv = NULL;
-
-	g_assert (CAMEL_IS_KOLAB_SESSION (self));
-	priv = CAMEL_KOLAB_SESSION_PRIVATE (self);
-
-	if (priv->passwd != NULL)
-		g_free (priv->passwd);
-
-	priv->passwd = g_strdup (passwd);
-}
-
 /*----------------------------------------------------------------------------*/
