@@ -138,6 +138,7 @@ kolab_mail_access_get_type (void) G_GNUC_CONST;
 
 gboolean
 kolab_mail_access_configure (KolabMailAccess *self,
+                             EBackend *backend,
                              KolabSettingsHandler *ksettings,
                              GError **err);
 
@@ -176,6 +177,7 @@ kolab_mail_access_query_folder_info_online (KolabMailAccess *self,
 gboolean
 kolab_mail_access_create_source (KolabMailAccess *self,
                                  const gchar *sourcename,
+                                 KolabFolderTypeID sourcetype,
                                  GCancellable *cancellable,
                                  GError **err);
 
