@@ -70,6 +70,8 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 
+#include <libebackend/libebackend.h>
+
 #include "kolab-mail-handle.h"
 #include "kolab-mail-mime-builder.h"
 #include "kolab-settings-handler.h"
@@ -104,6 +106,7 @@ kolab_mail_imap_client_get_type (void) G_GNUC_CONST;
 
 gboolean
 kolab_mail_imap_client_configure (KolabMailImapClient *self,
+                                  EBackend *backend,
                                   KolabSettingsHandler *ksettings,
                                   KolabMailMimeBuilder *mimebuilder,
                                   GError **err);
@@ -159,6 +162,7 @@ kolab_mail_imap_client_query_folder_summary (KolabMailImapClient *self,
 gboolean
 kolab_mail_imap_client_create_folder (KolabMailImapClient *self,
                                       const gchar *foldername,
+                                      KolabFolderTypeID foldertype,
                                       GCancellable *cancellable,
                                       GError **err);
 
