@@ -446,7 +446,7 @@ kolab_util_backend_account_uid_new_from_settings (KolabSettingsHandler *ksetting
 	}
 
 	if (esource_uid != NULL) {
-		tmp_str = g_strdup (esource_uid);
+		account_uid = g_strdup (esource_uid);
 		goto done;
 	}
 
@@ -460,11 +460,11 @@ kolab_util_backend_account_uid_new_from_settings (KolabSettingsHandler *ksetting
 
 	tmp_str = g_strconcat (user, "@", host, NULL);
 
- done:
-
 	account_uid = g_strconcat (KOLAB_CAMEL_PROVIDER_PROTOCOL, "__", tmp_str, NULL);
 	g_free (tmp_str);
 
+ done:
+
 	return account_uid;
 }
 
