@@ -30,8 +30,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
-#include <libebook/e-contact.h>
-#include <libecal/e-cal-component.h>
+#include <libebook/libebook.h>
+#include <libecal/libecal.h>
 
 /*
  * some kolab constants
