@@ -58,7 +58,8 @@ typedef enum {
 	/* Book-IMAPX (EBook-Backend) */
 	KOLAB_FOLDER_TYPE_CONTACT,		/* 15 */
 	KOLAB_FOLDER_TYPE_CONTACT_DEFAULT,	/* 16 */
-	KOLAB_FOLDER_LAST_TYPE
+	KOLAB_FOLDER_LAST_TYPE,
+	KOLAB_FOLDER_TYPE_AUTO = KOLAB_FOLDER_TYPE_INVAL
 } KolabFolderTypeID;
 
 /* IMAP folder annotation: /vendor/kolab/incidences-for
