@@ -52,6 +52,8 @@
 #define AUTH "auth"
 #define SECURITY "security"
 
+#define TESTFOLDERNAME "INBOX/__test_kolab_mail_access_create_delete_source"
+
 /*----------------------------------------------------------------------------*/
 
 static gboolean
@@ -294,6 +296,35 @@ test_kolab_mail_access_query_folder_info (KolabMailAccess *kmailaccess,
 	return TRUE;
 }
 
+static gboolean
+test_kolab_mail_access_create_delete_source (KolabMailAccess *kmailaccess,
+                                             GError **err)
+{
+	GError *tmp_err = NULL;
+	gboolean ok = FALSE;
+
+	ok = kolab_mail_access_create_source (kmailaccess,
+	                                      TESTFOLDERNAME,
+	                                      KOLAB_FOLDER_TYPE_EMAIL,
+	                                      NULL,
+	                                      &tmp_err);
+	if (! ok) {
+		g_propagate_error (err, tmp_err);
+		return FALSE;
+	}
+
+	ok = kolab_mail_access_delete_source (kmailaccess,
+	                                      TESTFOLDERNAME,
+	                                      NULL,
+	                                      &tmp_err);
+	if (! ok) {
+		g_propagate_error (err, tmp_err);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 test_kolab_mail_access (KolabFolderContextID context)
 {
@@ -334,7 +365,9 @@ test_kolab_mail_access (KolabFolderContextID context)
 	kmailaccess = KOLAB_MAIL_ACCESS (g_object_new (KOLAB_TYPE_MAIL_ACCESS, NULL));
 
 	/* configure mail access subsystem */
-	ok = kolab_mail_access_configure (kmailaccess, ksettings, &tmp_err);
+	/* FIXME Need to pass an EBackend here.
+	 *       As currently written, this call will abort. */
+	ok = kolab_mail_access_configure (kmailaccess, NULL, ksettings, &tmp_err);
 	if (! ok)
 		goto test_part_cleanup;
 
@@ -358,6 +391,12 @@ test_kolab_mail_access (KolabFolderContextID context)
 	                                               &tmp_err);
 	if (! ok)
 		goto test_part_cleanup;
+	
+	/* test folder creation and deletion */
+	ok = test_kolab_mail_access_create_delete_source (kmailaccess,
+	                                                  &tmp_err);
+	if (! ok)
+		goto test_part_cleanup;
 
 	/* synchronize metadata and caches with server */
 	ok = kolab_mail_access_synchronize (kmailaccess,
